/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.PageContext;


/**
 * <p>
 * This interface defines the contract for classes wishing to discover the current node. Implementations of this
 * interface will be called when the current page has not been defined (ie a page scope attribute for
 * <code>BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE</code>). The implementation should discover, in it's own way, the
 * current node when <code>getCurrentNode()</code> is called and return the node representation of that url.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> Implementations should be thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public interface NodeDiscovery {
    /**
     * <p>
     * Retrieves the node which represents the current page. If the current page could not be found or is not
     * applicable - <code>null</code> will be returned. The returned node should include any query strings available.
     * </p>
     *
     * @param pageContext A non-null page context to use.
     *
     * @return A non-null node representing the current url or <code>null</code> if the node couldn't be determined.
     *
     * @throws IllegalArgumentException if the <code>pageContext</code> is <code>null</code>.
     * @throws BreadCrumbException if an exception occurs discovering the page.
     */
    BreadCrumbNode getCurrentNode(PageContext pageContext) throws BreadCrumbException;
}
