/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import org.apache.cactus.JspTestCase;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;


/**
 * <p>
 * Tests functionality and error cases of <code>AbstractNodeDiscovery</code> class. A mock class will be created for
 * testing since <code>AbstractNodeDiscovery</code> is an abstract class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class AbstractNodeDiscoveryUnitTest extends JspTestCase {
    /** Represents the <code>AbstractNodeDiscovery</code> instance used for testing. */
    private AbstractNodeDiscovery nodeDiscovery = null;

    /**
     * <p>
     * Sets up the test environment. The test instance is created.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    protected void setUp() throws Exception {
        nodeDiscovery = new MockAbstractNodeDiscovery();
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>AbstractNodeDiscovery()</code>.
     * </p>
     */
    public void testAbstractNodeDiscovery_Accuracy() {
        assertNotNull("The AbstractNodeDiscovery instance should be created.", nodeDiscovery);
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code>.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_Accuracy() throws Exception {
        nodeDiscovery.setPageContext(this.pageContext);
        assertEquals("DoStartTag failed", Tag.SKIP_BODY, nodeDiscovery.doStartTag());

        // The current node should be saved as an attribute into the current page context using page scope.
        Object object = this.pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE,
                PageContext.PAGE_SCOPE);
        assertTrue("The current node should be saved as an attribute into the current page context using page scope.",
            object instanceof BreadCrumbNode);
        assertEquals("The current node should be saved properly.", "title", ((BreadCrumbNode) object).getTitle());
        assertEquals("The current node should be saved properly.", "url", ((BreadCrumbNode) object).getUrl());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getTitle()</code>.
     * </p>
     */
    public void testGetTitle_Accuracy() {
        assertEquals("The title should be got properly.", null, nodeDiscovery.getTitle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setTitle(String)</code> with <code>null</code> title.
     * </p>
     */
    public void testSetTitle_NullTitle() {
        nodeDiscovery.setTitle(null);
        assertEquals("The title should be set properly.", null, nodeDiscovery.getTitle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setTitle(String)</code> with empty title.
     * </p>
     */
    public void testSetTitle_EmptyTitle() {
        nodeDiscovery.setTitle(" ");
        assertEquals("The title should be set properly.", null, nodeDiscovery.getTitle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setTitle(String)</code>.
     * </p>
     */
    public void testSetTitle_Accuracy() {
        String title = "title";
        nodeDiscovery.setTitle(title);
        assertEquals("The title should be set properly.", title, nodeDiscovery.getTitle());
    }
}
