/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.DynamicPathDiscovery;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.NodeMatcher;

import org.apache.cactus.JspTestCase;
import org.apache.cactus.WebResponse;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.BodyTagSupport;


/**
 * <p>
 * Tests functionality and error cases of <code>BreadCrumbTrailTag</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class BreadCrumbTrailTagUnitTest extends JspTestCase {
    /** Represents the <code>BreadCrumbTrailTag</code> instance used for testing. */
    private BreadCrumbTrailTag tag = null;

    /** Represents the <code>BreadCrumbNode</code> instances used for testing. */
    private List nodes = null;

    /**
     * <p>
     * Sets up the test environment. Adds the default config file. The test instance is created.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    protected void setUp() throws Exception {
        UnitTestHelper.addConfig();
        tag = new BreadCrumbTrailTag();

        UnitTestHelper.addConfig("ObjectFactoryConfig.xml");

        BreadCrumbNode[] nodesArray = new BreadCrumbNode[3];
        nodesArray[0] = new BreadCrumbNode("rootNodeUrl", "rootNodeTitle");
        nodesArray[1] = new BreadCrumbNode("NodeUrl", "NodeTitle");
        nodesArray[2] = new BreadCrumbNode("currentNodeUrl", "currentNodeTitle");
        nodes = Arrays.asList(nodesArray);
    }

    /**
     * Removes the configuration if it exists.
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        UnitTestHelper.clearConfig();
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code>.
     * </p>
     */
    public void testBreadCrumbTrailTag_Accuracy() {
        assertNotNull("The BreadCrumbTrailTag instance should be created.", tag);
        assertEquals("The pathSeparator should be set properly.", ":",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathSeparator"));
        assertEquals("The pathSeparatorStyle should be set properly.", "pathSeparatorStyle",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathSeparatorStyle"));
        assertEquals("The nodeStyle should be set properly.", "nodeStyle",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "nodeStyle"));
        assertEquals("The rootNodeStyle should be set properly.", "rootNodeStyle",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "rootNodeStyle"));
        assertEquals("The currentNodeStyle should be set properly.", "currentNodeStyle",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "currentNodeStyle"));
        assertEquals("The mouseOverStyle should be set properly.", "mouseOverStyle",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "mouseOverStyle"));
        assertEquals("The pathDirection should be set properly.", "0",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathDirection").toString());
        assertEquals("The silentErrors should be set properly.", false,
            Boolean.valueOf(UnitTestHelper.getPrivateField(tag.getClass(), tag, "silentErrors").toString())
                   .booleanValue());
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the namespace doesnot exist, the
     * default property values should be loaded.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_NotExistNamespaceAccuracy() throws Exception {
        UnitTestHelper.clearConfig();
        tag = new BreadCrumbTrailTag();
        assertNotNull("The BreadCrumbTrailTag instance should be created.", tag);
        assertEquals("The pathSeparator should be set properly.", ":",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathSeparator"));
        assertEquals("The pathSeparatorStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathSeparatorStyle"));
        assertEquals("The nodeStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "nodeStyle"));
        assertEquals("The rootNodeStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "rootNodeStyle"));
        assertEquals("The currentNodeStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "currentNodeStyle"));
        assertEquals("The mouseOverStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "mouseOverStyle"));
        assertEquals("The pathDirection should be set properly.", "0",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathDirection").toString());
        assertEquals("The silentErrors should be set properly.", false,
            Boolean.valueOf(UnitTestHelper.getPrivateField(tag.getClass(), tag, "silentErrors").toString())
                   .booleanValue());
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the pathSeparator is missing.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_MissingPathSeparatorAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_MissingPathSeparator.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The pathSeparator should be set properly.", ":",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathSeparator"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the pathSeparator is empty.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_EmptyPathSeparatorAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_EmptyPathSeparator.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The pathSeparator should be set properly.", " ",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathSeparator"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the pathSeparatorStyle is missing.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_MissingPathSeparatorStyleAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_MissingPathSeparatorStyle.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The pathSeparatorStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathSeparatorStyle"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the pathSeparatorStyle is empty.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_EmptyPathSeparatorStyleAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_EmptyPathSeparatorStyle.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The pathSeparatorStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathSeparatorStyle"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the nodeStyle is missing.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_MissingNodeStyleAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_MissingNodeStyle.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The nodeStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "nodeStyle"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the nodeStyle is empty.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_EmptyNodeStyleAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_EmptyNodeStyle.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The nodeStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "nodeStyle"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the currentNodeStyle is missing.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_MissingCurrentNodeStyleAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_MissingCurrentNodeStyle.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The currentNodeStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "currentNodeStyle"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the currentNodeStyle is empty.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_EmptyCurrentNodeStyleAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_EmptyCurrentNodeStyle.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The currentNodeStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "currentNodeStyle"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the rootNodeStyle is missing.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_MissingRootNodeStyleAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_MissingRootNodeStyle.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The rootNodeStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "rootNodeStyle"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the rootNodeStyle is empty.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_EmptyRootNodeStyleAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_EmptyRootNodeStyle.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The rootNodeStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "rootNodeStyle"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the mouseOverStyle is missing.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_MissingMouseOverStyleAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_MissingMouseOverStyle.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The mouseOverStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "mouseOverStyle"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the mouseOverStyle is empty.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_EmptyMouseOverStyleAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_EmptyMouseOverStyle.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The mouseOverStyle should be set properly.", null,
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "mouseOverStyle"));
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the pathDirection is missing.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_MissingPathDirectionAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_MissingPathDirection.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The pathDirection should be set properly.", "0",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathDirection").toString());
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the pathDirection is invalid.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_InvalidPathDirectionAccuracy1() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_InvalidPathDirection1.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The pathDirection should be set properly.", "0",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathDirection").toString());
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the pathDirection is invalid.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_InvalidPathDirectionAccuracy2() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_InvalidPathDirection2.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The pathDirection should be set properly.", "0",
            UnitTestHelper.getPrivateField(tag.getClass(), tag, "pathDirection").toString());
    }

    /**
     * <p>
     * Tests the accuracy of constructor: <code>BreadCrumbTrailTag()</code> when the silentErrors is missing.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testBreadCrumbTrailTag_MissingSilentErrorsAccuracy() throws Exception {
        UnitTestHelper.addConfig("BreadCrumbTrailTagConfig_MissingSilentErrors.xml");
        tag = new BreadCrumbTrailTag();
        assertEquals("The silentErrors should be set properly.", false,
            Boolean.valueOf(UnitTestHelper.getPrivateField(tag.getClass(), tag, "silentErrors").toString())
                   .booleanValue());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPathDirection()</code>.
     * </p>
     */
    public void testGetPathDirection_Accuracy() {
        assertEquals("The pathDirection should be got properly.", 0, tag.getPathDirection());
    }

    /**
     * <p>
     * Tests the method <code>setPathDirection(int)</code> when the pathDirection is invalid, IllegalArgumentException
     * is expected.
     * </p>
     */
    public void testSetPathDirection_InvalidPathDirection() {
        try {
            tag.setPathDirection(-1);
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setPathDirection(int)</code>.
     * </p>
     */
    public void testSetPathDirection_Accuracy() {
        tag.setPathDirection(1);
        assertEquals("The pathDirection should be set properly.", 1, tag.getPathDirection());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPathSeparator()</code>.
     * </p>
     */
    public void testGetPathSeparator_Accuracy() {
        assertEquals("The pathSeparator should be got properly.", ":", tag.getPathSeparator());
    }

    /**
     * <p>
     * Tests the method <code>setPathDirection(String)</code> when the pathSeparator is null, IllegalArgumentException
     * is expected.
     * </p>
     */
    public void testSetPathSeparator_NullPathSeparator() {
        try {
            tag.setPathSeparator(null);
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setPathSeparator(String)</code> when the pathSeparator is empty.
     * </p>
     */
    public void testSetPathSeparator_EmptyPathSeparator() {
        tag.setPathSeparator(" ");
        assertEquals("The pathSeparator should be set properly.", " ", tag.getPathSeparator());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setPathSeparator(String)</code>.
     * </p>
     */
    public void testSetPathSeparator_Accuracy() {
        tag.setPathSeparator("#");
        assertEquals("The pathSeparator should be set properly.", "#", tag.getPathSeparator());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPathSeparatorStyle()</code>.
     * </p>
     */
    public void testGetPathSeparatorStyle_Accuracy() {
        assertEquals("The pathSeparatorStyle should be got properly.",
                "pathSeparatorStyle", tag.getPathSeparatorStyle());
    }

    /**
     * <p>
     * Tests the method <code>setPathSeparatorStyle(String)</code> when the pathSeparatorStyle is null.
     * </p>
     */
    public void testSetPathSeparatorStyle_NullPathSeparatorStyle() {
        tag.setPathSeparatorStyle(null);
        assertEquals("The pathSeparatorStyle should be set properly.", null, tag.getPathSeparatorStyle());
    }

    /**
     * <p>
     * Tests the method <code>setPathSeparatorStyle(String)</code> when the pathSeparatorStyle is empty.
     * </p>
     */
    public void testSetPathSeparatorStyle_EmptyPathSeparatorStyle() {
        tag.setPathSeparatorStyle(" ");
        assertEquals("The pathSeparatorStyle should be set properly.", null, tag.getPathSeparatorStyle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setPathSeparatorStyle(String)</code>.
     * </p>
     */
    public void testSetPathSeparatorStyle_Accuracy() {
        tag.setPathSeparatorStyle("#");
        assertEquals("The pathSeparatorStyle should be set properly.", "#", tag.getPathSeparatorStyle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getNodeStyle()</code>.
     * </p>
     */
    public void testGetNodeStyle_Accuracy() {
        assertEquals("The nodeStyle should be got properly.", "nodeStyle", tag.getNodeStyle());
    }

    /**
     * <p>
     * Tests the method <code>setNodeStyle(String)</code> when the nodeStyle is null.
     * </p>
     */
    public void testSetNodeStyle_NullNodeStyle() {
        tag.setNodeStyle(null);
        assertEquals("The nodeStyle should be set properly.", null, tag.getNodeStyle());
    }

    /**
     * <p>
     * Tests the method <code>setNodeStyle(String)</code> when the nodeStyle is empty.
     * </p>
     */
    public void testSetNodeStyle_EmptyNodeStyle() {
        tag.setNodeStyle(" ");
        assertEquals("The nodeStyle should be set properly.", null, tag.getNodeStyle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setNodeStyle(String)</code>.
     * </p>
     */
    public void testSetNodeStyle_Accuracy() {
        tag.setNodeStyle("#");
        assertEquals("The nodeStyle should be set properly.", "#", tag.getNodeStyle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getCurrentNodeStyle()</code>.
     * </p>
     */
    public void testGetCurrentNodeStyle_Accuracy() {
        assertEquals("The currentNodeStyle should be got properly.", "currentNodeStyle", tag.getCurrentNodeStyle());
    }

    /**
     * <p>
     * Tests the method <code>setCurrentNodeStyle(String)</code> when the currentNodeStyle is null.
     * </p>
     */
    public void testSetCurrentNodeStyle_NullCurrentNodeStyle() {
        tag.setCurrentNodeStyle(null);
        assertEquals("The currentNodeStyle should be set properly.", null, tag.getCurrentNodeStyle());
    }

    /**
     * <p>
     * Tests the method <code>setCurrentNodeStyle(String)</code> when the currentNodeStyle is empty.
     * </p>
     */
    public void testSetCurrentNodeStyle_EmptyCurrentNodeStyle() {
        tag.setCurrentNodeStyle(" ");
        assertEquals("The currentNodeStyle should be set properly.", null, tag.getCurrentNodeStyle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setCurrentNodeStyle(String)</code>.
     * </p>
     */
    public void testSetCurrentNodeStyle_Accuracy() {
        tag.setCurrentNodeStyle("#");
        assertEquals("The currentNodeStyle should be set properly.", "#", tag.getCurrentNodeStyle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getRootNodeStyle()</code>.
     * </p>
     */
    public void testGetRootNodeStyle_Accuracy() {
        assertEquals("The rootNodeStyle should be got properly.", "rootNodeStyle", tag.getRootNodeStyle());
    }

    /**
     * <p>
     * Tests the method <code>setRootNodeStyle(String)</code> when the rootNodeStyle is null.
     * </p>
     */
    public void testSetRootNodeStyle_NullRootNodeStyle() {
        tag.setRootNodeStyle(null);
        assertEquals("The rootNodeStyle should be set properly.", null, tag.getRootNodeStyle());
    }

    /**
     * <p>
     * Tests the method <code>setRootNodeStyle(String)</code> when the rootNodeStyle is empty.
     * </p>
     */
    public void testSetRootNodeStyle_EmptyRootNodeStyle() {
        tag.setRootNodeStyle(" ");
        assertEquals("The rootNodeStyle should be set properly.", null, tag.getRootNodeStyle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setRootNodeStyle(String)</code>.
     * </p>
     */
    public void testSetRootNodeStyle_Accuracy() {
        tag.setRootNodeStyle("#");
        assertEquals("The rootNodeStyle should be set properly.", "#", tag.getRootNodeStyle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getMouseOverStyle()</code>.
     * </p>
     */
    public void testGetMouseOverStyle_Accuracy() {
        assertEquals("The mouseOverStyle should be got properly.", "mouseOverStyle", tag.getMouseOverStyle());
    }

    /**
     * <p>
     * Tests the method <code>setMouseOverStyle(String)</code> when the mouseOverStyle is null.
     * </p>
     */
    public void testSetMouseOverStyle_NullMouseOverStyle() {
        tag.setMouseOverStyle(null);
        assertEquals("The mouseOverStyle should be set properly.", null, tag.getMouseOverStyle());
    }

    /**
     * <p>
     * Tests the method <code>setMouseOverStyle(String)</code> when the mouseOverStyle is empty.
     * </p>
     */
    public void testSetMouseOverStyle_EmptyMouseOverStyle() {
        tag.setMouseOverStyle(" ");
        assertEquals("The mouseOverStyle should be set properly.", null, tag.getMouseOverStyle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setMouseOverStyle(String)</code>.
     * </p>
     */
    public void testSetMouseOverStyle_Accuracy() {
        tag.setMouseOverStyle("#");
        assertEquals("The mouseOverStyle should be set properly.", "#", tag.getMouseOverStyle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getPathOverride()</code>.
     * </p>
     */
    public void testGetPathOverride_Accuracy() {
        assertEquals("The pathOverride should be got properly.", null, tag.getPathOverride());
    }

    /**
     * <p>
     * Tests the method <code>setPathOverride(String)</code> when the pathOverride is null.
     * </p>
     */
    public void testSetPathOverride_NullPathOverride() {
        tag.setPathOverride(null);
        assertEquals("The pathOverride should be set properly.", null, tag.getPathOverride());
    }

    /**
     * <p>
     * Tests the method <code>setPathOverride(String)</code> when the pathOverride is empty.
     * </p>
     */
    public void testSetPathOverride_EmptyPathOverride() {
        tag.setPathOverride(" ");
        assertEquals("The pathOverride should be set properly.", null, tag.getPathOverride());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setPathOverride(String)</code>.
     * </p>
     */
    public void testSetPathOverride_Accuracy() {
        tag.setPathOverride("#");
        assertEquals("The pathOverride should be set properly.", "#", tag.getPathOverride());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getSilentErrors()</code>.
     * </p>
     */
    public void testGetSilentErrors_Accuracy() {
        assertEquals("The silentErrors should be got properly.", false, tag.getSilentErrors());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setSilentErrors(boolean)</code>.
     * </p>
     */
    public void testSetSilentErrors_Accuracy() {
        tag.setSilentErrors(true);
        assertEquals("The silentErrors should be set properly.", true, tag.getSilentErrors());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>getTitle()</code>.
     * </p>
     */
    public void testGetTitle_Accuracy() {
        assertEquals("The title should be got properly.", null, tag.getTitle());
    }

    /**
     * <p>
     * Tests the method <code>setTitle(String)</code> when the title is null.
     * </p>
     */
    public void testSetTitle_NullTitle() {
        tag.setTitle(null);
        assertEquals("The title should be set properly.", null, tag.getTitle());
    }

    /**
     * <p>
     * Tests the method <code>setTitle(String)</code> when the title is empty.
     * </p>
     */
    public void testSetTitle_EmptyTitle() {
        tag.setTitle(" ");
        assertEquals("The title should be set properly.", null, tag.getTitle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>setTitle(String)</code>.
     * </p>
     */
    public void testSetTitle_Accuracy() {
        tag.setTitle("#");
        assertEquals("The title should be set properly.", "#", tag.getTitle());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doInitBody()</code>. It is the rootNode.
     * </p>
     */
    public void testDoInitBody_Accuracy1() {
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        tag.doInitBody();

        // check the result
        assertEquals("The node should be correct.", nodes.get(0),
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("The style should be correct.", "rootNodeStyle",
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, PageContext.PAGE_SCOPE));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doInitBody()</code>. It is the node.
     * </p>
     */
    public void testDoInitBody_Accuracy2() {
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        tag.doInitBody();

        // check the result
        assertEquals("The node should be correct.", nodes.get(1),
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("The style should be correct.", "nodeStyle",
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, PageContext.PAGE_SCOPE));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doInitBody()</code>. It is the currentNode.
     * </p>
     */
    public void testDoInitBody_Accuracy3() {
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        tag.doInitBody();

        // check the result
        assertEquals("The node should be correct.", nodes.get(2),
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("The style should be correct.", "currentNodeStyle",
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, PageContext.PAGE_SCOPE));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doInitBody()</code>. It is the currentNode.
     * </p>
     */
    public void testDoInitBody_Accuracy4() {
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        tag.doInitBody();

        // check the result
        assertEquals("The node should be correct.", nodes.get(2),
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("The style should be correct.", "currentNodeStyle",
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, PageContext.PAGE_SCOPE));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doInitBody()</code>. It is the node.
     * </p>
     */
    public void testDoInitBody_Accuracy5() {
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        tag.doInitBody();

        // check the result
        assertEquals("The node should be correct.", nodes.get(1),
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("The style should be correct.", "nodeStyle",
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, PageContext.PAGE_SCOPE));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doInitBody()</code>. It is the rootNode.
     * </p>
     */
    public void testDoInitBody_Accuracy6() {
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        tag.doInitBody();

        // check the result
        assertEquals("The node should be correct.", nodes.get(0),
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("The style should be correct.", "rootNodeStyle",
            pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, PageContext.PAGE_SCOPE));
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the rootNode.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_WithBodyContentAccuracy1() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        bodyContent.print("content");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_WithBodyContentAccuracy1(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "content<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_WithBodyContentAccuracy2() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        bodyContent.print("content");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_WithBodyContentAccuracy2(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "content<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the current node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_WithBodyContentAccuracy3() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        bodyContent.print("content");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_WithBodyContentAccuracy3(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "content<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It has no next node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_WithBodyContentAccuracy4() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        bodyContent.print("content");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        iter.next();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.SKIP_BODY, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_WithBodyContentAccuracy4(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "content";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the current node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_WithBodyContentAccuracy5() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        bodyContent.print("content");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_WithBodyContentAccuracy5(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "content<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_WithBodyContentAccuracy6() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        bodyContent.print("content");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_WithBodyContentAccuracy6(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "content<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the root node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_WithBodyContentAccuracy7() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        bodyContent.print("content");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_WithBodyContentAccuracy7(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "content<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It has no previous node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_WithBodyContentAccuracy8() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        bodyContent.print("content");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        iter.previous();
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.SKIP_BODY, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_WithBodyContentAccuracy8(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "content";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the rootNode.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_EmptyBodyContentAccuracy1() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        bodyContent.print(" ");

        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should not be cleared.", " ", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_EmptyBodyContentAccuracy1(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_EmptyBodyContentAccuracy2() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        bodyContent.print(" ");

        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should not be cleared.", " ", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_EmptyBodyContentAccuracy2(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the current node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_EmptyBodyContentAccuracy3() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();

        bodyContent.print(" ");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should not be cleared.", " ", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_EmptyBodyContentAccuracy3(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It has no next node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_EmptyBodyContentAccuracy4() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();

        bodyContent.print(" ");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        iter.next();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.SKIP_BODY, tag.doAfterBody());
        assertEquals("The bodyContent should not be cleared.", " ", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_EmptyBodyContentAccuracy4(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the current node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_EmptyBodyContentAccuracy5() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();

        bodyContent.print(" ");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should not be cleared.", " ", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_EmptyBodyContentAccuracy5(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_EmptyBodyContentAccuracy6() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();

        bodyContent.print(" ");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should not be cleared.", " ", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_EmptyBodyContentAccuracy6(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the root node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_EmptyBodyContentAccuracy7() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();

        bodyContent.print(" ");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should not be cleared.", " ", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_EmptyBodyContentAccuracy7(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It has no previous node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_EmptyBodyContentAccuracy8() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();

        bodyContent.print(" ");
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        iter.previous();
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.SKIP_BODY, tag.doAfterBody());
        assertEquals("The bodyContent should not be cleared.", " ", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_EmptyBodyContentAccuracy8(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the rootNode.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_NoBodyContentAccuracy1() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_NoBodyContentAccuracy1(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_NoBodyContentAccuracy2() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_NoBodyContentAccuracy2(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the current node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_NoBodyContentAccuracy3() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_NoBodyContentAccuracy3(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It has no next node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_NoBodyContentAccuracy4() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);

        ListIterator iter = nodes.listIterator();
        iter.next();
        iter.next();
        iter.next();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.SKIP_BODY, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_NoBodyContentAccuracy4(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the current node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_NoBodyContentAccuracy5() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_NoBodyContentAccuracy5(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_NoBodyContentAccuracy6() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_NoBodyContentAccuracy6(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It is the root node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_NoBodyContentAccuracy7() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_BODY_AGAIN, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_NoBodyContentAccuracy7(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span class=path-separator-style>:</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doAfterBody()</code>. It has no previous node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoAfterBody_NoBodyContentAccuracy8() throws Exception {
        BodyContent bodyContent = this.pageContext.pushBody();
        tag.setBodyContent(bodyContent);
        tag.setPageContext(this.pageContext);
        tag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);

        ListIterator iter = nodes.listIterator(nodes.size());
        iter.previous();
        iter.previous();
        iter.previous();
        UnitTestHelper.setPrivateField(tag.getClass(), tag, "iterator", iter);

        assertEquals("DoAfterBody failed", BodyTagSupport.SKIP_BODY, tag.doAfterBody());
        assertEquals("The bodyContent should be cleared.", "", bodyContent.getString());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoAfterBody_NoBodyContentAccuracy8(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doEndTag()</code>. It has no previous node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoEndTag_Accuracy1() throws Exception {
        tag.setPageContext(this.pageContext);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_PAGE, tag.doEndTag());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoEndTag_Accuracy1(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "</span>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doEndTag()</code>. It has no previous node.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoEndTag_Accuracy2() throws Exception {
        UnitTestHelper.clearConfig();
        tag = new BreadCrumbTrailTag();
        tag.setPageContext(this.pageContext);

        assertEquals("DoAfterBody failed", BodyTagSupport.EVAL_PAGE, tag.doEndTag());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoEndTag_Accuracy2(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoStartTag_NoCurrentPathAccuracy() throws Exception {
        tag.setPageContext(this.pageContext);

        // set the current node
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("E");
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, currentNode, PageContext.PAGE_SCOPE);

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->B->C->E, the path should be A->B->C->E
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D"), UnitTestHelper.buildNodeMatcher("E")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), new ArrayList());

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        assertEquals("DoStartTag failed", BodyTagSupport.EVAL_BODY_BUFFERED, tag.doStartTag());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoStartTag_NoCurrentPathAccuracy(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<span>\n" + "<style>\n" + "path-separator-style {pathSeparatorStyle}\n"
            + ".node-style a:link {nodeStyle}\n" + ".current-node-style a:link {currentNodeStyle}\n"
            + ".root-node-style a:link {rootNodeStyle}\n" + ".node-style a:hover {mouseOverStyle}\n"
            + ".current-node-style a:hover {mouseOverStyle}\n" + ".root-node-style a:hover {mouseOverStyle}\n"
            + "</style>\n";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoStartTag_NoStyleAccuracy() throws Exception {
        tag = new BreadCrumbTrailTag();
        tag.setPageContext(this.pageContext);

        // set the current node
        BreadCrumbNode currentNode = UnitTestHelper.buildBreadCrumbNode("E");
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, currentNode, PageContext.PAGE_SCOPE);

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->B->C->E, the path should be A->B->C->E
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D"), UnitTestHelper.buildNodeMatcher("E")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), new ArrayList());

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        assertEquals("DoStartTag failed", BodyTagSupport.EVAL_BODY_BUFFERED, tag.doStartTag());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoStartTag_NoStyleAccuracy(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code>.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    public void testDoStartTag_NoCurrentNodeAccuracy() throws Exception {
        tag.setPageContext(this.pageContext);

        // build the siteMap
        Map siteMap = new HashMap();

        // the map will be A->B->C->D, A->B->C->E, the path should be A->B->C->E
        siteMap.put(UnitTestHelper.buildNodeMatcher("A"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("B")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("B"),
            Arrays.asList(new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("C")}));
        siteMap.put(UnitTestHelper.buildNodeMatcher("C"),
            Arrays.asList(
                new NodeMatcher[] {UnitTestHelper.buildNodeMatcher("D"), UnitTestHelper.buildNodeMatcher("E")}));

        siteMap.put(UnitTestHelper.buildNodeMatcher("D"), new ArrayList());
        siteMap.put(UnitTestHelper.buildNodeMatcher("E"), new ArrayList());

        this.pageContext.setAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, siteMap, PageContext.SESSION_SCOPE);

        assertEquals("DoStartTag failed", BodyTagSupport.SKIP_BODY, tag.doStartTag());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoStartTag_NoCurrentNodeAccuracy(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }
}
