/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import org.apache.cactus.JspTestCase;
import org.apache.cactus.WebResponse;

import java.util.HashMap;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;


/**
 * <p>
 * Tests functionality and error cases of <code>HtmlAnchorNodeFormatterTag</code> class.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class HtmlAnchorNodeFormatterTagUnitTest extends JspTestCase {
    /** Represents the <code>HtmlAnchorNodeFormatterTag</code> instance used for testing. */
    private HtmlAnchorNodeFormatterTag formatter = null;

    /**
     * <p>
     * Sets up the test environment. The test instance is created.
     * </p>
     *
     * @throws Exception any exception to junit.
     */
    protected void setUp() throws Exception {
        formatter = new HtmlAnchorNodeFormatterTag();
    }

    /**
     * <p>
     * Tests the accuracy of constructor <code>HtmlAnchorNodeFormatterTag()</code>.
     * </p>
     */
    public void testHtmlAnchorNodeFormatterTag_Accuracy() {
        assertNotNull("The HtmlAnchorNodeFormatterTag instance should be created.", formatter);
    }

    /**
     * <p>
     * Tests the method <code>doStartTag()</code> when the BreadCrumbNode is missing, expect BreadCrumbException.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_MissingBreadCrumbNode() throws Exception {
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "style",
                PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);

        try {
            formatter.doStartTag();
            fail("BreadCrumbException should be thrown.");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>doStartTag()</code> when the BreadCrumbNode is not valid type, expect
     * BreadCrumbException.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_NotValidBreadCrumbNode() throws Exception {
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, "node", PageContext.PAGE_SCOPE);
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "style",
                PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);

        try {
            formatter.doStartTag();
            fail("BreadCrumbException should be thrown.");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the method <code>doStartTag()</code> when the style attribute is not valid type, expect
     * BreadCrumbException.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_NotValidStyle() throws Exception {
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, new BreadCrumbNode("url", "title"),
            PageContext.PAGE_SCOPE);
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, new HashMap(),
            PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);

        try {
            formatter.doStartTag();
            fail("BreadCrumbException should be thrown.");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code> when the title for BreadCrumbNode is missing.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_MissingTitleAccuracy() throws Exception {
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, new BreadCrumbNode("url", null),
            PageContext.PAGE_SCOPE);
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "style",
                PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);
        assertEquals("DoStartTag failed", Tag.SKIP_BODY, formatter.doStartTag());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoStartTag_MissingTitleAccuracy(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<a class=\"style\" href=\"url\">Unspecified</a>";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code> when the style attribute is an empty string.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_EmptyStyleAccuracy() throws Exception {
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, new BreadCrumbNode("url", "title"),
            PageContext.PAGE_SCOPE);
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "  ", PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);
        assertEquals("DoStartTag failed", Tag.SKIP_BODY, formatter.doStartTag());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoStartTag_EmptyStyleAccuracy(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<a href=\"url\">title</a>";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code> when the style attribute is missing.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_MissingStyleAccuracy() throws Exception {
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, new BreadCrumbNode("url", "title"),
            PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);
        assertEquals("DoStartTag failed", Tag.SKIP_BODY, formatter.doStartTag());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoStartTag_MissingStyleAccuracy(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<a href=\"url\">title</a>";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }

    /**
     * <p>
     * Tests the accuracy of method <code>doStartTag()</code>.
     * </p>
     *
     * @throws Exception any exception to JUnit.
     */
    public void testDoStartTag_Accuracy() throws Exception {
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, new BreadCrumbNode("url", "title"),
            PageContext.PAGE_SCOPE);
        this.pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "style",
                PageContext.PAGE_SCOPE);
        formatter.setPageContext(this.pageContext);
        assertEquals("DoStartTag failed", Tag.SKIP_BODY, formatter.doStartTag());
    }

    /**
     * <p>
     * Verifies the result.
     * </p>
     *
     * @param theResponse the WebResponse instance in cacuts, return from server.
     *
     * @throws Exception any exception to JUnit.
     */
    public void endDoStartTag_Accuracy(WebResponse theResponse) throws Exception {
        // the expected result.
        String expected = "<a class=\"style\" href=\"url\">title</a>";
        assertEquals("The webResponse result should be correct.", expected, theResponse.getText());
    }
}
